/*******************************************************************************



    RELGIO_FALANTE.C

    Modo PWM - utiliza o PWM interno do pic a 19,53 khz.

    Autor: Claudio Lrios

    Data: 03/12/2014

    Microcontrolador: PIC 16f628a
    RTC: DS1307
    FLASH: 25L1606E  (usada como 'BIOS' de notebook,pc,roteadores,etc)
    Sada de som: LM386

    Objetivo: Relgio sem display, mas reproduzindo voz humana para
    informar as horas. Uso para pessoas cegas.

    Parte integrante do blog 'LARIOS.TECNOLOGIA.WS'
    Proposta didtica apenas, sendo testada somente em placa de protoboard.

******************************************************************************/


#include <16F628A.h>
#use delay(clock=20000000)//USE XTAL DE 20MHZ
#fuses NOWDT,HS, NOPUT, NOPROTECT, BROWNOUT, NOMCLR, NOLVP, NOCPD
#use fast_io(a)
#use fast_io(b)
#ZERO_RAM
#rom 0x2100 = {0,0,0}

#byte trisa= 0x85
#byte trisb= 0x86
#byte porta= 0x05
#byte portb= 0x06
#byte pr2=   0x92
#byte ccp1l= 0x15

#bit    cs           = 0x06.2  //pino 8 rb2
#bit    miso         = 0x06.1  //pino 7 rb1
#bit    mosi         = 0x06.4  //pino 10 rb4
#bit    slk          = 0x06.5  //pino 11 rb5
#bit    flagt2       = 0x0C.1  //flag do timer 2
#bit    som          = 0x06.3  //pino 9 saida de som
#bit    tris_som     = 0x86.3  //sentido
#bit    sda          = 0x06.0  //rb0 pino 6
#bit    tris_sda     = 0x86.0  //rb0 sentido
#bit    scl          = 0x06.7  //rb0 pino 13
#bit    sw_ouvir     = 0x05.5  //ra0 pino4    ouvir horas/calendrio
#bit    sw_hr        = 0x05.2  //ra2 pino 1   ajustar horas
#bit    sw_min       = 0x05.3  //ra3 pino 2   ajustar minutos
#bit    sw_rel       = 0x05.0  //ra0 pino 17  ajustar relogio
#bit    sw_desp      = 0x05.1  //ra0 pino 18  ajustar despertador
#bit    sw_ldesp     = 0x06.6  //rb6 pino 12 on/off despertador
#bit    sw_cal       = 0x05.4  //ra4 pino 3  ajuste do calendrio
int8 time[7]={0,0,0,0,0,0,0};//armazena as variveis do DS1307 aps sua leitura
int8  valor;
int1 flagh,flagp;
int1 fadjr,fadjr1,fadjr2,fadjd,fadjd1,fadjd2,df,dd,fadjc,fadjc1,fadjc2,fadjc3,fadjc4;
/*==============================================================================

                      ROTINAS DE COMUNICAO SPI

==============================================================================*/


/*==============================================================================
     GERA SINAL DE CLOCK PARA A FLASH
==============================================================================*/

void clock(void){ slk=1; delay_us(1);slk=0; delay_us(1);}

/*==============================================================================
    ENVIA 8 BITS PARA A FLASH
==============================================================================*/
void env8(int8 zbuf1){
  int8 a3;
  for(a3=0;a3<8;a3++){
  if(bit_test(zbuf1,7)) mosi=1; else mosi=0;
   clock();
   zbuf1<<=1;
  }
}
/*==============================================================================
     RECEBE 8 BITS DA FLASH
==============================================================================*/

int8 recebe_flash_8bits(void){
 int8 a5,b;
 b=0;
 for (a5=0;a5<8;a5++){
  shift_left(&b,1,miso); //original
 // shift_right(&b,1,miso);//teste
  clock();
 }
   return (b);
}


/*==============================================================================
     ROTINA DE REPRODUO DE SOM GRAVADO NA MEMRIA FLASH EXTERNA
==============================================================================*/

void reproduzir(int32 address){
 int8 buf;

  cs=0;
  env8(0x03);//comando de reproduzir em modo contnuo
  env8(make8(address,2));//pega endereo inicial
  env8(make8(address,1));
  env8(make8(address,0));
  tris_som=0;//port de som como sada
  while(1){// reproduz at encontrar 0x22 no cdigo
      buf=recebe_flash_8bits(); //recebe byte
      if(buf==0x22) {/*tris_som=1; */cs=1;break;} //para no fim do arquivo
      if((!sw_ldesp)&&(df)){dd=1;break;}//para ao apertar boto do alarme
      while(!flagt2) ;//aguarda 'pwm' finalizar processo
      ccp1l=buf;//carga do valor do pwm
      flagt2=0;//zera flag tmr2
  }
  cs=1;
}



void fala_zero(){
 reproduzir(0);//zero
}
void fala_um(){
 //reproduzir(0x10b81a);//um
 reproduzir(0x3180);//um
}
void fala_uma(){
 reproduzir(0x48b8);//um
}
void fala_dois(){
reproduzir(0x65aa);//dois
}
void fala_duas(){
 reproduzir(0x9e97);//zero
}
void fala_tres(){
 reproduzir(0xd223);//tres
}

void fala_quatro(){
reproduzir(0xfe1d);//quatro
}

void fala_cinco(){
reproduzir(0x12d71);
}

void fala_seis(){
 reproduzir(0x16150);
}

void fala_sete(){
 reproduzir(0x199c2);
}

void fala_oito(){
 reproduzir(0x1cdc2);
}

void fala_nove(){
 reproduzir(0x1fbeb);
 }

void fala_dez(){
 reproduzir(0x22dd6);
}

void fala_onze(){
 reproduzir(0x25f4c);
}

void fala_doze(){
 reproduzir(0x28a9d);
}

void fala_treze(){
 reproduzir(0x2be42);
}

void fala_quatorze(){
 reproduzir(0x2eb33);
}

void fala_quinze(){
 reproduzir(0x32676);
}

void fala_dezesseis(){
 reproduzir(0x35c09);
}

void fala_dezessete(){
 reproduzir(0x3aa4a);
}

void fala_dezoito(){
 reproduzir(0x3f2e8);
}

void fala_dezenove(){
 reproduzir(0x431e0);
}

void fala_vinte(){
 reproduzir(0x46fee);
}

void fala_trinta(){
 reproduzir(0x49f34);
}

void fala_quarenta(){
 reproduzir(0x4bfe8);
}

void fala_cinquenta(){
 reproduzir(0x4f30c);
}

void fala_sessenta(){
 reproduzir(0x535d3);
}

void fala_setenta(){
 reproduzir(0x57685);
}

void fala_oitenta(){
 reproduzir(0x5b375);
}

void fala_noventa(){
 reproduzir(0x5eb5e);
}

void fala_e(){
  reproduzir(0x10803e);
}


void fala_segundos(){
  if(flagp)reproduzir(0xacfd6);else reproduzir(0xa9069);
}


void fala_minutos(){
  if(flagp)reproduzir(0xa555a);else reproduzir(0xa2946);
}


void fala_horas(){
  if(flagp)reproduzir(0x9fac2);else reproduzir(0x9dd8f);
}
// pega um byte e o reproduz

void pega_num(int8 val){
switch (val){
    case 0: fala_zero();break;
    case 1: {if(!flagh){fala_um();break;} else {fala_uma();break;}}
    case 2: {if(!flagh){fala_dois();break;} else {fala_duas();break;}}
    case 3: fala_tres();break;
    case 4: fala_quatro();break;
    case 5: fala_cinco();break;//cinco
    case 6: fala_seis();break;//seis
    case 7: fala_sete();break;//sete
    case 8: fala_oito();break;//oito
    case 9: fala_nove();break;//nove
    case 0x10: fala_dez();break;//dez
    case 0x11: fala_onze();break;//onze
    case 0x12: fala_doze();break;//doze
    case 0x13: fala_treze();break;//treze
    case 0x14: fala_quatorze();break;//quatorze
    case 0x15: fala_quinze();break;//quinze
    case 0x16: fala_dezesseis();break;//dezesseis
    case 0x17: fala_dezessete();break;//dezessete
    case 0x18: fala_dezoito();break;//dezoito
    case 0x19: fala_dezenove();break;//dezenove
    case 0x20: fala_vinte();break;//vinte
    case 0x30: fala_trinta();break;//trinta
    case 0x40: fala_quarenta();break;//quarenta
    case 0x50: fala_cinquenta();break;//cinquenta
    case 0x60: fala_sessenta();break;//sessenta
    case 0x70: fala_setenta();break;//setenta
    case 0x80: fala_oitenta();break;//oitenta
    case 0x90: fala_noventa();break;//noventa
    }
}


void fala_byte(int8 valor){//reproduz um byte
int8 z;
     if(valor>0x20){
     z=valor;
     z=z&0xf0;//pega nibble alto
     pega_num(z);
     z=valor&0x0f;
     if(z){ //se for zero no reproduz o 'zero'
     fala_e();
     z=valor;
     z=z&0x0f;//pega nibble low
     pega_num(z);
     }
    }
    else{
    pega_num(valor);
    }

}


void fala_dia(){
 reproduzir(0x11f93d);
}

void fala_ano(){
 reproduzir(0x125fd2);
}

void fala_mes(){
 reproduzir(0x123beb);
}

void fala_anos(){
 int8 v;
      flagh=0;//excesses masculinas (um,dois)
      v=time[6];
      if(v<2)flagp=0; else flagp=1;
      reproduzir(0xc76b1);//'dois mil e'
      fala_byte(v);
      
 }
void fala_hr(){
int8 v;
      flagh=1;//excesses femininas (uma,duas)
      v=time[2];
      if(!v){reproduzir(0x86e1b);}//& meia noite
      else{//&
      if(v<2)flagp=0; else flagp=1;
      fala_byte(v);
      fala_horas();
      }//&
}

void fala_mn(){
int8 v;
      flagh=0;//excesses masculinas (um,dois)
      v=time[1];
      if(v<2)flagp=0; else flagp=1;
      fala_byte(v);
      fala_minutos();
    //  fala_e();

}
 void fala_sg(){
 int8 v;
      flagh=0;//excesses masculinas (um,dois)
      v=time[0];
      if(v<2)flagp=0; else flagp=1;
      fala_byte(v);
      fala_segundos();
 }

void fala_meses(){
int8 v;
      flagh=0;//excesses masculinas (um,dois)
      v=time[5];
      if(v<2)flagp=0; else flagp=1;
    //  fala_mes();
      switch (v){
    case 1:{reproduzir(0x162c5b); break;}  //jan
    case 2:{reproduzir(0x1660ff); break;}//fev
    case 3:{reproduzir(0x16946e); break;}//mar
    case 4:{reproduzir(0x16bb56); break;}//abr
    case 5:{reproduzir(0x16dbae); break;}//mai
    case 6:{reproduzir(0x170294); break;}//jun
    case 7:{reproduzir(0x172d63); break;}//jul
    case 8:{reproduzir(0x175832); break;}//ago
    case 9:{reproduzir(0x178ad3); break;}//set
    case 0x10:{reproduzir(0x17bb86); break;}//out
    case 0x11:{reproduzir(0x17ea40); break;}//nov
    case 0x12:{reproduzir(0x1820c8); break;}//dez 
    }
    // fala_byte(v);
}



void fala_diasemana(){

int8 v;
      flagh=0;//excesses masculinas (um,dois)
      v=time[3];
      if(v<2)flagp=0; else flagp=1;
    //  fala_mes();
      switch (v){
    case 1:{reproduzir(0x10cef4); break;}//domingo
    case 2:{reproduzir(0x10faa0); break;}//segunda
    case 3:{reproduzir(0x11296c); break;}//tera
    case 4:{reproduzir(0x115065); break;}//quarta
    case 5:{reproduzir(0x1172ac); break;}//quinta
    case 6:{reproduzir(0x1197bd); break;}//sexta
    case 7:{reproduzir(0x11d23f); break;}//sabado
   }
   if((v>1)&&(v<0x07)) reproduzir(0x137029);//feira
   else{delay_ms(200);}//excesso domingo e sabado
}
void fala_datas(){
int8 v;
      flagh=0;//excesses masculinas (um,dois)
      v=time[4];//dia
      if(v<2)flagp=0; else flagp=1;
      fala_dia();
      fala_byte(v);

}



void fala_time(){
   fala_hr();//fala as horas
   fala_mn();//fala os minutos
   fala_sg();//fala os segundos
}


//==============================================================================
//                     ROTINAS PARA DS1307
//==============================================================================


void delay_m(void){ delay_us(5);}

VOID I2C_START(VOID){//INICIA COMUNICAO 'START'

    sda=1;
    delay_m();
    scl=1;
    delay_m();
    sda=0;
    delay_m();
    scl=0;
}

void I2C_STOP(void){//ENCERRA COMUNICAO I2C
    sda=0;
    delay_m();
    scl=1;
    delay_m();
    sda=1;
    delay_m();
    scl=0;
}


void clk(void){//CLOCK
   delay_m();
   scl=1;
   delay_m();
   scl=0;
}


void ACK(void){//envia 'ack'
   sda=0;
   clk();
 }

void NO_ACK(void){//envia 'no ack'
   sda=1;
   clk();
}

void I2C_WRITE(int8 valor){//ESCREVER 8 BITS NO DS1307
 int8 a;
  for(a=0;a<8;a++){

   if(bit_test(valor,7)){sda=1;} else{sda=0;}
   clk();
   valor=valor<<1;
  }
}

int8 I2C_READ(void){ //LER 8 BITS DO DS1307
int8 c=0;
int8 a;
  tris_sda=1;
  for(a=0;a<8;a++){
   c=c<<1;
   if(sda) c++;
   clk();

  }
  tris_sda=0;
  return (c);
}



void ds_write(int8 adr,int8 valor){//escreve 8 bits no endereo informado
  I2C_START();
  I2C_WRITE(0xd0);
  ACK();
  I2C_WRITE(adr);
  ACK();
  I2C_WRITE(valor);
  ACK();
  I2C_STOP();
}

int8 ds_read(int8 adr){//le 8 bits do DS1307 no endereo informado
 int8 a;
  I2C_START();
  I2C_WRITE(0xd0);
  ACK();
  I2C_WRITE(adr);
  ACK();
  I2C_START();
  I2C_WRITE(0xd1);
  ACK();
  a=I2C_READ();
  NO_ACK();
  I2C_STOP();
  RETURN (a);
}

void ds_read8(void){//ler todos os registradores de tempo
 int8 a;
  I2C_START();
  I2C_WRITE(0xd0);
  ACK();
  I2C_WRITE(0);
  ACK();
  I2C_START();
  I2C_WRITE(0xd1);
  ACK();
  for(a=0;a<7;a++){
  time[a]=I2C_READ();
  if(a==6)no_ack();else ACK();
  }
  I2C_STOP();

}

void ds_write8(void){//escreve todos os registradores de tempo
 int8 a;
  I2C_START();
  I2C_WRITE(0xd0);
  ACK();
  I2C_WRITE(0);
  for(a=0;a<7;a++){
    ACK();
    I2C_WRITE(time[a]);
  }
  ACK();
  I2C_STOP();
}

//incrementa e converte byte (hexa para decimal)
int8 inc_reg(int8 reg1){
  int8 a,b;
  a=reg1;
  b=a;
  b=swap(b)&0x0f;
  a=a&0x0f;
  if(++a>9) {a=0; ++b;}//incrementa e testa
  return (swap(b)|a);//retorna em decimal
}


/*==============================================================================

                      ROTINA MAIN

==============================================================================*/

void main() {

   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_timer_2 ( T2_DIV_BY_1, 0, 1);
   setup_comparator(NC_NC_NC_NC);
   setup_ccp1(CCP_PWM);
   setup_vref(FALSE);
   trisa=0b11111111;
   trisb=0b01001010;
   pr2=0xff;//periodo max
   cs=1;
   delay_ms(500);
   ds_read8(); //pega dados do ds1307
   if(time[0]&0x80) {time[0]&=0x7f;ds_write8();}//liga DS1307 se necessrio

/*==============================================================================

                      LOOP PRINCIPAL

==============================================================================*/


 

 while(1){
    ds_read8(); //pega dados do ds1307 e coloca em time[]

    if(read_eeprom(2)){//compara se deve despertar
      if((read_eeprom(0)==time[2])&&(read_eeprom(1)==time[1])){
      df=1;
      if(!dd){
       if(time[2]<0x18){reproduzir(0x830e0);}//'bom dia'
       else {
        reproduzir(0x7f197);}//'boa noite'
        reproduzir(0xb1937);//'So'
        fala_time();//informa as horas atuais
        reproduzir(0xbe201);//'ja esta na hora de levantar'
       }
      }
      else {df=0;dd=0;}
    }

    if(!sw_ouvir){//informa a hora atual
    ds_read8(); //pega dados do ds1307 e coloca em time[]
    reproduzir(0xb1937);//'So'
    if(sw_ouvir){ fala_time();}//toque rpido fala somente as horas
    else{//toque mais demorado
    fala_time();//fala a hora + calendrio
    fala_diasemana(); //dia da semana
    delay_ms(200);
    fala_datas();//dia
    reproduzir(0x1618ff);//de
    fala_meses();//mes
    reproduzir(0x1618ff);//de
    fala_anos();//ano (dois mil e...)
    }
    }
 //============================================================
 //calendario
     if(!sw_cal){
       if(!fadjc) reproduzir(0x127732);//'Ajustando calendrio'
       fadjc=1;
       if(!sw_hr){
         if(!fadjc1){
          reproduzir(0x133797);//'Ajustar'
          reproduzir(0x1219f8);//'data'
          fadjc1=1;}
          while(!sw_hr){
           time[4]=inc_reg(time[4]);
           if(time[4]>0x31) {time[4]=1;}//limita a 31
           fala_datas();//informa alterao
      }
      ds_write8();//grava DS1307
      }

       if(!sw_min){
         if(!fadjc2){
          reproduzir(0x133797);//'Ajustar'
          reproduzir(0x123beb);//'mes'
          fadjc2=1;}
          while(!sw_min){
           time[5]=inc_reg(time[5]);
           if(time[5]>0x12) {time[5]=1;}//limita a 12
           fala_meses();//informa alterao
      }
      ds_write8();//grava DS1307
      }

       if(!sw_desp){
         if(!fadjc3){
          reproduzir(0x133797);//'Ajustar'
          reproduzir(0x125fd2);//'ano'
          fadjc3=1;}
          while(!sw_desp){
           time[6]=inc_reg(time[6]);
           if(time[6]>0x99) {time[6]=0;}//limita a 99
           fala_anos();//informa alterao
      }
        ds_write8();//grava DS1307
    }


      if(!sw_rel){
         if(!fadjc4){
          reproduzir(0x159e0b);//'dia da semana'
          fadjc4=1;}
          while(!sw_rel){
           time[3]=inc_reg(time[3]);
           if(time[3]>0x07) {time[3]=1;}//limita a 7
           fala_diasemana();//informa alterao
      }
        ds_write8();//grava DS1307
    }



    }





//=============================================================
    if((!sw_rel)&&(sw_desp)&&(sw_cal)){
       if(!fadjr) reproduzir(0x6a3e8);//'Ajustar relgio'
       fadjr=1;
       if(!sw_hr){
         if(!fadjr1)reproduzir(0x7113d);//'Ajustando horas
          fadjr1=1;
          while(!sw_hr){
           time[2]=inc_reg(time[2]);
           if(time[2]>0x23) {time[2]=0;}//limita a 23
           fala_hr();//informa alterao
      }
      ds_write8();//grava DS1307
      }

       if(!sw_min){
         if(!fadjr2)reproduzir(0x77a68);//'Ajustando minutos
          fadjr2=1;
          while(!sw_min){
           time[1]=inc_reg(time[1]);
           if(time[1]>0x59) {time[1]=0;}//limita a 59
           fala_mn();//informa alterao
      }
      ds_write8();//grava DS1307
      }

    }

    //===============================


     if((!sw_desp)&&(sw_rel)&&(sw_cal)){
       if(!fadjd) reproduzir(0x62542);//'Ajustar despertador'
       fadjd=1;
       if(!sw_hr){
         if(!fadjd1)reproduzir(0x7113d);//'Ajustando horas
          fadjd1=1;
          while(!sw_hr){
           time[2]=inc_reg(time[2]);
           if(time[2]>0x23) {time[2]=0;}//limita a 23
           fala_hr();//informa alterao
      }
      write_eeprom(0,time[2]);//salva na eeprom
      }

       if(!sw_min){
         if(!fadjr2)reproduzir(0x77a68);//'Ajustando minutos
          fadjd2=1;
          while(!sw_min){
           time[1]=inc_reg(time[1]);
           if(time[1]>0x59) {time[1]=0;}//limita a 59
           fala_mn();//informa alterao
      }
      write_eeprom(1,time[1]);//salva na eeprom
      }

    }

    if(sw_rel) {fadjr=0; fadjr1=0;fadjr2=0;}//desliga flags travas
    if(sw_desp){fadjd=0;fadjd1=0;fadjd2=0;}
    if(sw_cal){ fadjc=0;fadjc1=0;fadjc2=0;fadjc3=0;fadjc4=0;}


    if((!sw_ldesp)&&(!read_eeprom(2))){
    reproduzir(0x976a0);
    write_eeprom(2,1);
    time[2]=read_eeprom(0);
    time[1]=read_eeprom(1);
    fala_hr();
    fala_mn();
    }//'alarme foi ligado'
    else{
     if((!sw_ldesp)&&(read_eeprom(2))){
     reproduzir(0x8f760);
     write_eeprom(2,0);;
     }//'alarme foi desligado'
    }
    delay_ms(100);

   }//while
}//main
